//**********************************************************************
// Control a AD9851 DDS based on the good work of others including:
// Mike Bowthorpe, http://www.ladyada.net/rant/2007/02/cotw-ltc6903/ and
// http://www.geocities.com/leon_heller/dds.html
// This code by Peter Marks http://marxy.org
// Modyfied for PIC18F4553 by OZ6YM, Palle A. Andersen, oz6ym@teknik.dk
//**********************************************************************
#include "DDS.h"
#include "p18f4553.h"
#include "delays.h"
#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#include "Porte.h"

// Kalibrerings-vrdi for njagtig frekvensjustering.
// DDS-60 har CLOCK-krystal-osc. p 30 MHz, og er opsat til
// at kre med intern multiplier ( 6 x 30 = 180000000 ).
// DDS-CLOCK kan justeres, sledes at en 
// evt. krystal-frekvens-afvigelse kan elemineres.
// I mit tilflde, skulle frekvensjusteringen sledes 
// ndre ved at fratrkke 2750 hz ( se herunder).
// ********************************************************

// EXTERN DEFINITIONER
 extern void udskrivStep(unsigned long frq);
// extern void DDSsweep(void);
// extern void DDSloop(void);
 extern void sendFrequency(unsigned long);// unsignet long "frequency"
 extern void byte_out(unsigned char);
 extern void outOne(void);
 extern void outZero(void);
 extern void KnapTest(void);

// Variabel-liste
// ********************************************************
int OUT = 0;
int LOW = 0;
int IN = 1;
int HIGH = 1;

unsigned long frequens;
unsigned long frequency;
unsigned long tuning_word;
int i;
// ********** END of DEFINITIONS **************************

// **************** Send out a ONE (high-bit) *************
// ****** Send en enkelt bit af BIT-TOGET p ialr 32 bit. *
// ********************************************************
void outOne(void)
{
DDSCLOCK = LOW;			// St Clock-port LOW
DDSDATA = HIGH;			// St DATA-port HIGH
DDSCLOCK = HIGH;		// St Clock-port HIGH
DDSDATA = LOW;			// St DATA-port LOW
}
// ***************** end of OUT-ONE ***********************

// ***************** Send out a ZERO (low-bit) ************
// ****** Send en enkelt bit af BIT-TOGET p ialr 32 bit. *
// ********************************************************
void outZero(void)
{
DDSCLOCK = LOW;			// St Clock-port LOW
DDSDATA = LOW;			// St DATA-port LOW
DDSCLOCK = HIGH;		// St Clock-port HIGH
}
// ***************** end of OUT-ZERO **********************

// ********* INITIALICE DDS-60 ****************************
// ********************************************************

// ** Set all portes DATA-RETNINGS-REGISTRE til OUTPUT ****
// ********************************************************
void DDSsetup(void)
{
DDSDATATRIS = OUT;
DDSCLOCKTRIS = OUT;
DDSLOADTRIS = OUT;
}
// ********************* DONE *****************************

// ****** Send en hel byte ud til DDS-60 ******************
// ********************************************************
void byte_out(unsigned char byte)
{
  for (i = 0; i < 8; i++)		// Byte = 8 bit
  {
    if ((byte & 1) == 1)
      outOne();
    else
      outZero();
    byte = byte >> 1;
  }
}
// ********************* DONE *****************************
// ********************************************************

// ********************************************************
// **** Dette er hoved-rutinen                          ***
// **** til at sende et FULDT VOGNTOG afsted til DDS-60 ***
// ********************************************************
void sendFrequency(unsigned long frequency)
{
	tuning_word = (frequency * pow(2, 32))/DDS_CLOCK;
	DDSLOAD = LOW;	
				// take load pin low
	for ( i = 0; i < 32; i++)		//frequens -> DDS
	{  	
		if ((tuning_word & 1) == 1)
			{
				outOne();
			}
    	else
			{
	   			outZero();
			}
		tuning_word = tuning_word >> 1;
	}
	byte_out(0x09);

	DDSLOAD = HIGH;					// Take load pin high again
}
//******************* END of SEND *************************




