// ***************************************************************
// SDR-radio VFO
// OZ6YM, Palle A. Andersen ( August 2009 ),
// revideret til PIC18F4553 den 25.12.2009
// ***************************************************************
// PORT-forbrug:  PIC16F4553 - konfigurationen:
//		port RB0  = Rotator A ( PullUp = 10 K Ohm til + 5 volt )
//		port RB1  = Rotator B ( PullUp = 10 K Ohm til + 5 volt )
//
// ===================================================
// 3 porte bruges til styring af DDS:
//      DDS               CPU
// ----------------------------------
//		DATA 	(pin3) 	= RC7
//		CLOCK 	(pin2)	= RB5
//		LOAD 	(pin1) 	= RA3
// Se i vrigt DDS.h vedr. kalibrering af frekvens-njagtighed.
// ===================================================
// LCD Display driver - STANDARD 4 line x 20 Char display
// Forbindelse mellem CPU = PIC18F4553 og LDC,
// 		defineret herunder, er:
//
//		LCD				CPU				SIGNAL
//      ---------------------------------------------------------
//		pin 4 - RS		=	pin 21 - RB0 = Register Select
//		pin 5 - R/W		=	pin 22 - RB1 - READ/WRITE
//		pin 6 - E		= 	pin 23 - RB2 - Enable
//		pin 7,8,9,10	=	NC			 - Not Connected
// 4 bit DATA
//		pin 11			=	pin 25 - RB4 - DATA
//		pin 12 			=   pin 26 - RB5 - DATA
//		pin 13			=   pin 27 - RB6 - DATA
// 		pin 14			= 	pin 28 - RB7 - DATA
// **************************************************************
// Flgende porte bruges som Bndvlger porte:
//		Knapper:		CPU-port;		State:
//      ---------------------------------------------------------
//		band1			=	RA0			output led 1	
//		band2			=	RA1			output led 2
//		band3			=	RA2			output led 3
//		band4			=	RA4			output led 4
// **************************************************************
// Flgende porte bruges som knapporte:
//      Synonym             Port        Digital       Funktion                          Lednings Farvekode:
//------------------------------------------------------------------------------------------------------
//		STEP_POINTER	=	RE0			input knap 1  Vlg STEP							Gul/Rd
//		BAND_POINTER	=	RE2			input knap 2  Vlg BAND							Gul/Hvid     OBS
//		zband			=	RC0			input knap 3  Zeroset current Band				Hvid/Gr
//		UPstep			=	RC1			input knap 4  Vlg Skift FRQ STEP UP			Bl/Grn
//		DOWNstep		=	RC2			input knap 5  Vlg Skift FRQ STEP DOWN			PINK
//		StandardVFO		=	RC6			input knap 6  Vlg Indls MF					GR
//		sweep			=	RB2			input knap 7  Vlg SWEEP StandardVFO/Sig.Gen.	Gr/Sort
//		esweep			=	RB4			input knap 8  Setuo MF Standard					Gul/Hvid     OBS
//		SWEEPband		=	RB3			input knap 9  Vlg Set SSB				Gul/Gr
//		scan			= 	RA5			input knap 10 STOP og GEM FRQ SIGNAL Gen.		Orange/Grn
//
// *****************************************************************************************************
// SYSTEM-INCLUDES
#include <string.h>
#include "p18f4553.h"
#include "delays.h"
#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#include "adc.h"
#include "eep.h"

// My Includes
#include "lcd.h"
#include "DDS.h"
#include "Porte.h"

// EXTERNE DEFINITIONER
extern void KnapTest(void);
extern void KnapScan(void);
extern void DDSsetup(void);
extern void sendFrequency(unsigned long);
extern void WRTpoint();
extern void udskrivMF(unsigned long);
extern void WRTstep(unsigned int);
extern void MyDelay(unsigned int del0);

extern unsigned long frequens;
extern unsigned long CurStartFrekvens;
extern unsigned long CurFrekvens;
extern unsigned long CurEndFrekvens;

// extern volatile signed char rotatorcount = 0;
extern signed char rotatorcount = 0;

extern signed short step; 
extern signed short step1;
extern signed short MyStep; 

extern int ksFlag;
extern int bestDB;
extern int menuflag;
extern int bandPTR;
extern int Cur_bandPTR;

// ******** Variabel-liste ********
unsigned long freq_frm;
unsigned long freq_to;
unsigned long freq_step;
unsigned long frq;		// variabel, mellemregning
unsigned long TMPfrekvens;
unsigned long MYfreq;
unsigned long TMP_ffrom;
unsigned long TMP_fto;
unsigned long CurStartFrekvens;
unsigned long CurFrekvens;
unsigned long CurEndFrekvens;
unsigned long HeroMF = 0;
unsigned long USB = 3000;
unsigned long LSB = -3000;
unsigned long CW = -1500;
unsigned long tmpSSB = 0;

int SSB = 0;

int knapON  = 0;
int knapOFF = 1;
int dl0;
int dell = 100; 				// delay-variabel
int menuflag;			// Flag, bruges i void Knaptest(void)
int TmpPTR;
int ch0, ch1, ch2, ch3, ch4, ch5, ch6, ch7; // 14025000
int inx;
int loopFlag = 0;
int Cur_bandPTR;
int bandPTR = 1;				  //  160 m   = bandPTR = 1,   INITIAL Start
					  			  //  080 m   = bandPTR = 2
								  //  060 m   = bandPTR = 3
					  			  //  040 m   = bandPTR = 4
					  			  //  030 m   = bandPTR = 5
								  //  020 m   = bandPTR = 6
								  //  SignalG = bandPTR = 7

// ***************************************************************
// *************  F R E K V E N S T A B E L  *********************
// ***************************************************************
unsigned long fs_160  =  7240000; // 7240000; // 160 m ->  1.820.000
unsigned long fc_160  =  7240000; // 7240000; // 160 m current frekvens
unsigned long fe_160  =  8000000; // 8000000; // 160 m ->  2.000.000

unsigned long fs_080  = 14000000; //  80 m ->  3.500.000
unsigned long fc_080  = 14000000; //  80 m current frekvens
unsigned long fe_080  = 15200000; //  80 m ->  3.800.000

unsigned long fs_060  = 20800000; //  60 m ->  5.200.000
unsigned long fc_060  = 20800000; //  60 m current frekvens
unsigned long fe_060  = 21600000; //  60 m ->  5.400.000

unsigned long fs_040  = 28000000; //  40 m ->  7.000.000
unsigned long fc_040  = 28000000; //  40 m current frekvens
unsigned long fe_040  = 28800000; //  40 m ->  7.200.000

unsigned long fs_030  = 40400000; //  30 m -> 10.100.000
unsigned long fc_030  = 40400000; //  30 m current frekvens
unsigned long fe_030  = 40600000; //  30 m -> 10.150.000

unsigned long fs_020  = 56000000; //  20 m -> 14.000.000
unsigned long fc_020  = 56000000; //  20 m current frekvens
unsigned long fe_020  = 57400000; //  20 m -> 14.350.000

unsigned long fs_Sig  = 20000;
unsigned long fc_Sig  = 0;
unsigned long fe_Sig  = 99999999;





// ***************************************************************
// ***********  DDS-60 UPDATE ************************************
// ***************************************************************
// ** UPDATE DDS-60 - Check at den totale bndgrnse for DDS-60
// ** ikke overskrides
// Denne rutine udskriver frekvensen p display, afhngiet af *****************
// om det er signalgenerator (bandPTR=7) eller et amatrbnd,(bandPTR=1-6) ****
// derefter checkes om det rigtige bndfilter og lysdiode er sat. *************
// ****************************************************************************
// ***************************************************************
void updateDDS(void)
{

// *************** udskriv kun SDRadio-bnd **********************
if ( bandPTR  < 7 )     
	{

// step1 = step fra rotator - freq_step et multiplikator, * 1,10,100... osv
// frequens = frekvens til DDS
// TMPfrekvens = der skal udskrives p display
// ************************************************************************
	if ((BAND_POINTER = knapOFF) && (STEP_POINTER = knapOFF)) {	
		frequens = frequens + step1  * freq_step;	// step1 fra ROTATOR
		}	
	TMPfrekvens = frequens / 4; 					// skal bruges til at vlge bndpas-filter
		
	if (TMPfrekvens <= 0) 							// har vi ramt bunden ?
		{ TMPfrekvens = 0;}
	}
else
	{    
// banmdPTR = 7 
// ********************* udskriv kun Signalgenerator/MF **********
	if ((BAND_POINTER = knapOFF) && (STEP_POINTER = knapOFF)) {	
		frequens =  frequens +  (step1/4) * freq_step;
		}											// send til DDS-60
	TMPfrekvens = frequens - HeroMF + tmpSSB;

	if	( HeroMF - TMPfrekvens  <= 0 )
		{ TMPfrekvens = 0; } 						//HeroMF + frequens; 
		
	if  ( TMPfrekvens < 0 )
		{ TMPfrekvens = 0;}

	if	( frequens + HeroMF >= 99999999 )			// < frequens )
		{ frequens = HeroMF; }	
	}	
		udskrivFRQ( TMPfrekvens ); 					// udskriv p display, Ja, NO devide
		sendFrequency(frequens);					// send til DDS-60
// SET BNDFILTER og LYSDIODER bestemt af current frequens-omrde
// ***************************************************************
PORTA = 0b00010111;								// sluk alle LEDS
if ( ( TMPfrekvens < 2500000 ) && ( TMPfrekvens > 0 ))      	{  PORTA = 0b00011110; }// led 1
if ( ( TMPfrekvens >= 2500000 )&& ( 5400000 > TMPfrekvens  )) 	{  PORTA = 0b00011101; }// led 2
if ( ( TMPfrekvens >= 5400000 ) && (  12500000 > TMPfrekvens ))	{  PORTA = 0b00011011; }// led 3
if ( ( TMPfrekvens >= 12500000 ) && ( 21500000 > TMPfrekvens ))	{  PORTA = 0b00001111; }// led 4
if ( ( TMPfrekvens >= 21500000 ) && ( 27900000 > TMPfrekvens ))	{   }
}

// END of updateDDS *************************************************

// **************************************
// Knap nr. 1 - Frekvens-step-skifte
// **************************************
void Stepskifte(void)
{
// ************* DEBAUNCE *****************
	if ( STEP_POINTER == knapON ) 					// er "fast"-knap trykket ?
	{	Delay10KTCYx(1);							// lad der g 10 ms

		if ( STEP_POINTER == knapON )				// stadig trykket efter 10 milisec. ?
// *********** END DEBAUNCE ***************
		{
			if ( 0 < step1 ) 						// er step1 positiv ?
			{ step == step++; }						// lg 1 til

			if ( 0 > step1 )						// er step1 negativ ?
			{ step == step--; }						// trk 1 fra
		}											// ellers afslut }
	}
	if ( step > 7 ) { step = 1; }					// step strre end 7    = 1
	if ( step <= 0 ) { step = 7; }					// eller mindre end 1   = 7

freq_step = (floor(pow(10,step)))/10;				// beregning af ROTATOR-step

// *** Skriv S=0X (step-pointer) p display ***
	ch1='0';										// tiere
	ch2='0';										// enere
while ( step >= 10 ) { step = step = 10; ch1++; } 	
		ch2 = step + 0x30; 					 		// rest enere

	sndlcd(0x54+0x80,'0');
	wrtlcdchr('S');
	wrtlcdchr('=');
	wrtlcdchr(ch1);
	wrtlcdchr(ch2);
// **** END " Skriv S=0X (step-pointer) p display ***
}
// ***************** END of Stepskifte ****************************************


// *******************************************
// ***  Knap 4 - DDS up **********************
// *** UP er trykket - update DDS-60
// *******************************************
void DDSup(void)
{
frequens = frequens + freq_step;
updateDDS();
}
// ***  END DDS up ********************************

// ************************************************
// ***  Knap: 5 -DDS down *************************
// *** DOWN er trykket - update DDS
// ************************************************
void DDSdown(void)
{
frequens = frequens - freq_step;
updateDDS();
}
// ***  END DDS down ******************************

// ***************************************************************
// ********************* BANDPOINTER *****************************
// *** Skriv B=0X (bandPTR) p display
// ***************************************************************
void WRTpoint(void)
{
	frq = bandPTR;
	ch1='0';											// tiere
	ch2='0';											// enere

while ( frq >= 10 ) { frq =	frq	- 10; ch1++; }
		ch2 = frq + 0x30;           					// rest enere
	sndlcd(0x40+0x80,'0');
	wrtlcdchr('B');
	wrtlcdchr('=');
	wrtlcdchr(ch1);
	wrtlcdchr(ch2);
}
// END of WRTpoint ***********************************************

// ********************************************************************
// *** Check/Gem frequens i FrekvensCurrent (fc) pointed by "bandPTR"
// *** Gem hvor du er kommet til, 
// *** nr der skiftes mellem programfunktioner VFO-A, VFO-B og VFO-C
// ********************************************************************
void StoCurrent(void) 
{
switch (bandPTR)
	{
	case 1: fc_160 = frequens; break;
	case 2: fc_080 = frequens; break;
    case 3: fc_060 = frequens; break;
	case 4: fc_040 = frequens; break;
	case 5: fc_030 = frequens; break;
	case 6: fc_020 = frequens; break;
	case 7: fc_Sig = frequens; break;
	}
}
// END of StoCurrent **************************************************

// ********************************************
// void KnapScan(void)
// SCANNING af knapper 1 (een) gang i passagen
// hvilken er evt. trykket ?
// STEP_POINTER = "ksFlag" sttes til 1 hvis "Vlg STEP" 
// BAND_POINTER = "ksFlag" sttes til 2 hvis "Bndvlger" 
// zband        = "ksFlag" sttes til 3 hvis "ZEROset alle variable"
// UPstep       = "ksFlag" sttes til 4 hvis "UPstep"
// DOWNstep     = "ksFlag" sttes til 5 hvis "DOWNstep"
// StandardVFO  = "ksFlag" sttes til 6 hvis "SET MF StandardVFO"
// sweep        = "ksFlag" sttes til 7 hvis "SWEEPband"
// SWEEPband    = "ksFlag" sttes til 8 hvis "Vlg MF i StandardVFO"
// esweep       = "ksFlag" sttes til 9 hvis "SSB setup"
// scan         = "ksFlag" sttes til 10 hvis "RESET StandardVFO"
// ********************************************
void KnapScan(void)
{ ksFlag == 0; 
// ********************************************
// STEP-POINTER-KNAP
// ********************************************
	if ( STEP_POINTER == knapON ) 					// er "fast"-knap trykket ?
	{
		Delay10KTCYx(dell);							// lad der g 10 ms
		if ( STEP_POINTER == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 1; }							// Ja, set flag
	}												// ellers afslut }
// ********************************************
// BAND-POINTER-KNAP
// *****************************************
	if ( BAND_POINTER == knapON )					// er "band"-knap sluppet ?
	{
		Delay10KTCYx(dell);
		if ( BAND_POINTER == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 2; }							// Ja, set flag
	}												// ellers afslut }        				// ellers afslut
// ********************************************
// RESET-CURRENT BAND - END OF SIGNALGENERATOR
// ********************************************
	if ( zband == knapON ) 							// er "sband"-knap trykket ?
	{
		Delay10KTCYx(dell);
		if ( zband == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 3; }							// Ja, set flag
	}												// ellers afslut }   
// ********************************************
// RECALL-KNAP - flyt frekvens TILBAGE
// ********************************************
	if ( UPstep == knapON ) 						// er "UP"-knap sluppet ?
	{
		Delay10KTCYx(10);
		if ( UPstep == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 4; }							// Ja, set flag
	}												// ellers afslut }   
// ********************************************
// DOWNstep-KNAP - flyt frekvens FREM
// ********************************************
	if ( DOWNstep == knapON )		 				// er "DOWN"-knap sluppet ?
	{
		Delay10KTCYx(10);
		if ( DOWNstep == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 5; }							// Ja, set flag
	}												// ellers afslut }   
// ********************************************
// MEM-ON-KNAP
// ********************************************
	if ( StandardVFO == knapON ) 					// er "MF-OFFset"-knap sluppet ?
	{
		Delay10KTCYx(dell);
		if ( StandardVFO == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 6; }							// Ja, set flag
	}												// ellers afslut }   
// ********************************************
// SWEEP-KNAP
// ********************************************
	if ( sweep == knapON ) 							// er knap 9 sluppet ?
	{
		Delay10KTCYx(dell);
		if ( sweep == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 7; }							// Ja, set flag
	}												// ellers afslut }
// ********************************************

// ********************************************
	if ( SWEEPband == knapON ) 						// er "SWEEP"-knap sluppet ?
	{
		Delay10KTCYx(dell);
		if ( SWEEPband == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 8; }							// Ja, set flag
	}												// ellers afslut }
// ********************************************
// Sweep TOOGLE knap
// ********************************************
	if ( esweep == knapON ) 						// er knap 8 sluppet ?
	{
		Delay10KTCYx(dell);
		if ( esweep == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 9; }							// Ja, set flag
	}												// ellers afslut }
// ********************************************
// RESET ALL knap
	if ( scan == knapON ) 							// er knap 10 sluppet ?
	{
		Delay10KTCYx(dell);
		if ( scan == knapON )				// stadig trykket efter 10 milisec. ?
		{	ksFlag = 10; }							// Ja, set flag
	}												// ellers afslut }
}
// ************** KnapScan ended **************

// ***************************************************************
// ***********     void set_bandPTR(void)              ***********
// ** med ndring af bandPTR skal frekvens ogs ndres          **
// ** opdaterer display og DDS                                  **
// ***************************************************************
void set_bandPTR(void)
{
switch (bandPTR)
	{
	case 0: break;		// do not exist
	case 1: 	
		freq_frm = CurStartFrekvens = fs_160;		// bandstart = bandPTR = 1 (160 m)
		freq_to = CurEndFrekvens = fe_160;			// band-end (160 m)

		if ( fc_160 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_160; }
		break;
	case 2:
		freq_frm = CurStartFrekvens = fs_080; 		// bandstart = bandPTR = 2 (080 m)
		freq_to = CurEndFrekvens = fe_080;			// band-end (080 m)

		if ( fc_080 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_080; }
		break;
	case 3:
		freq_frm = CurStartFrekvens = fs_060; 		// bandstart = bandPTR = 3 (060 m)
		freq_to = CurEndFrekvens = fe_060;			// band-end (080 m)

		if ( fc_060 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_060; }
		break;
	case 4:
		freq_frm = CurStartFrekvens = fs_040;		// bandstart = bandPTR = 4 (040 m)
		freq_to = CurEndFrekvens = fe_040;			// band-end (040 m)
	
		if ( fc_040 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_040; }
		break;
	case 5:
		freq_frm = CurStartFrekvens = fs_030;		// bandstart = bandPTR = 5 (030 m)
		freq_to = CurEndFrekvens = fe_030;			// band-end (030 m)

		if ( fc_030 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_030; }
		break;
	case 6:
		freq_frm = CurStartFrekvens = fs_020;		// bandstart = bandPTR = 6 (020 m)
		freq_to = CurEndFrekvens = fe_020;			// band-end (020 m)

		if ( fc_020 == 0 ) { frequens = freq_frm;}
		else { frequens = fc_020; }
		break;
	case 7:
		freq_frm = CurStartFrekvens = fs_Sig;		// Generator start - bandstart = bandPTR = 7
		freq_to = CurEndFrekvens = fe_Sig;			// Generator END

		if ( fc_Sig == 0 ) { frequens = fs_Sig; }
		else { frequens = fc_Sig; }
		break;
		}
	WRTpoint();										// Udskriv bandPTR p display og tnd lysdiode
}
// ***********     END of void set_bandPTR(void)        **********

// ***************************************************************
// ZERO-set det enkelte frekvensbnds current frekvens, baseret **
// p bandPTR, og g til start af dette bnd                    **
// ***************************************************************
void zeroCurrent(void)
{
switch (bandPTR)
	{
	case 1: fc_160 = fs_160; frequens = fc_160;TMP_ffrom = frequens;TMP_fto = fe_160; break;
	case 2: fc_080 = fs_080; frequens = fc_080;TMP_ffrom = frequens;TMP_fto = fe_080; break;
	case 3: fc_060 = fs_060; frequens = fc_060;TMP_ffrom = frequens;TMP_fto = fe_060; break;
	case 4: fc_040 = fs_040; frequens = fc_040;TMP_ffrom = frequens;TMP_fto = fe_040; break;
	case 5: fc_030 = fs_030; frequens = fc_030;TMP_ffrom = frequens;TMP_fto = fe_030; break;
	case 6: fc_020 = fs_020; frequens = fc_020;TMP_ffrom = frequens;TMP_fto = fe_020; break;
	case 7: fc_Sig = fs_Sig; frequens = fc_Sig;TMP_ffrom = frequens;TMP_fto = fe_Sig; break;
	}
}
// ***********     END of void zeroCurrent(void)         *********

// *********************************************************************************
// knap nr 7
//                      S W E E P   B A N D
// Denne funktion virker kun i SDRadio, og Signalgenerator, men IKKE i STANDARD VFO
// Hvis man bruger MF p display-linie 3, 
// skal man huske at tillgge denne underliggende MF-frekvens til
// displayfrekvensen p display-linie 2 under sweep.
// *********************************************************************************
void MySweep(void) 									// ksFlag = 7 nr programmet kommer hertil
{
	zeroCurrent();
/*************************** DEBAUNCE ************************/
	if ( sweep == knapON ) 							// er knap 9 sluppet ?
	{
		Delay10KTCYx(dell);
		if ( sweep == knapON )						// stadig trykket efter 10 milisec. ?
		{	ksFlag = 7; }							// Ja, set flag
	}												// ellers afslut }
/*************************** END DEBAUNCE ********************/
// hvis loopen krer ! ( toggle-funktion )	
	if 	( loopFlag > 0 )
		{ loopFlag = 0; 
		  goto EndOfSweep;							// stop den! hop ud af programmet
	}
/*************************************************************/

	ksFlag = 0;		 								// Nej
	sndlcd(0x00+0x80,'0');							// move LCD to pos 0
	wrtrlcd((char rom *) "VFO=C   SWEEP band  ",0);
	loopFlag = ksFlag;
	
	if ( HeroMF > 0 )
	{ 
		sndlcd(0x00+0x80,'0');							// move LCD to pos 0
		wrtrlcd((char rom *) "Ingen SWEEP med MF!!",0);
		loopFlag = 0;
		MyDelay(1000);
		udskrivMF(HeroMF);
		goto EndOfSweep;
	}
// ************************  START LOOP ------------------------------	
for (frequens = TMP_ffrom; frequens < TMP_fto; frequens = frequens + freq_step )
	{
	updateDDS();
	MyDelay(10);

	if ( STEP_POINTER = knapON ) 					// er "step"-knap trykket ?
		{ Stepskifte();
		}											// Ja, avancer step
	if ( sweep == knapON ) 							// er knap 9 sluppet ?
		{
			Delay10KTCYx(1);
			if ( sweep == knapON )					// stadig trykket efter 10 milisec. ?
			{
				goto EndOfSweep;						// Ja, set flag
			}
		}											// ellers afslut }
	}	  												// END OF LOOP ------------------
//********************************************************************
EndOfSweep:;
	MyDelay(20);
	loopFlag = 0;
	sndlcd(0x00+0x80,'0');							// move LCD to pos 0
	wrtrlcd((char rom *) "VFO=C   SWEEP END   ",0); 
}
// END of MySweep **********************************************************************

// ************************************************************
// ** Knap nr. 8 - Signal generator funktion
// frekvens fra 3 KHz til 99,999.999 MHz
// -----------------------------------------
// Som Standard VFO kan VFO'en bestryge samme omrde 
// Over 60 MHz med begrnste ydeevne
//    Vil alts osse kunne bruges p 6 meter bndet
// ************************************************************
void SigGen(void)     
{
StoCurrent();										//gem current frekvens
Cur_bandPTR = bandPTR;								//gem current bandPTR
													// Nu ved vi hvor vi kom fra

sndlcd(0x00+0x80,'0');								// move LCD to pos 0
wrtrlcd((char rom *) "VFO-C SIG. GENERATOR",0);
bandPTR = 7;										// set NY bandPTR
WRTpoint();											// skriv "B=07" p display
set_bandPTR();										// indls START og SLUT frekvens
}
// END of Signal generator ***************************************

// ***************************************************************
// void setFrekvens(void)
// Vlg amatrbnd, startfrekvens, slutfrekvens, Current frekvens
// Alt sttes ved hjlp af "bandPTR"
// ***************************************************************
void setFrekvens(void)
{
	StoCurrent();									// start with STORE Current Frekvens
if (bandPTR == 7)
{
	sndlcd(0x00+0x80,'0');								// move LCD to pos 0
	wrtrlcd((char rom *) "VFO-C SIG. GENERATOR",0);
	wrtrlcd((char rom *) "                    ",2);
	}
	else 
	{
	sndlcd(0x00+0x80,'0');							// move LCD to pos 0
	wrtrlcd((char rom *) "VFO-A   SDRadio VFO ",0);
	wrtrlcd((char rom *) "                    ",2);
	}

//********************************************************************************
	if ( BAND_POINTER == knapON ) 					// er "fast"-knap trykket ?
	{

		if ( BAND_POINTER == knapON )				// stadig trykket efter 10 milisec. ?
		{
			if ( 0 < step1 ) {bandPTR == bandPTR++; }	//+ step = 1 til 7;
			if ( 0 > step1 ) {bandPTR == bandPTR--;	}
		}											// ellers afslut }
	}
	if ( bandPTR > 6 ) { bandPTR = 1; }
	if ( bandPTR <= 0 ) { bandPTR = 6; }
//********************************************************************************
	set_bandPTR();
}
// ***********  END of setFrekvens(void)               ***********

// ************************************************************
// Dette stopper VFO-tilstand
// ************************************************************
void EndSigGen(void)
{
frequens = frequens - HeroMF;
StoCurrent();										// gem current frekvens
bandPTR = 1;						 				// Restore bandPTR
	sndlcd(0x00+0x80,'0');							// move LCD to pos 0
	wrtrlcd((char rom *) "VFO-A   SDRadio VFO ",0);
	wrtrlcd((char rom *) "                    ",2);
set_bandPTR();
HeroMF = 0;
SSB = 0;
tmpSSB = 0;
}

// ** END of Signal-generator**************************************

// ****************************************************************
// Konverter og udskriv frekvensen p display
//*****************************************************************
void wrtDEC(unsigned long frq)
{
  ch0='0';		// ti MG
  ch1='0';		// MG
  ch2='0';    	// hun tus        
  ch3='0';    	// ti tus
  ch4='0';    	// tus
  ch5='0';    	// hundrede
  ch6='0';		// tier
  ch7='0';		// ener
  while (frq>=10000000)
	{ frq=frq-10000000; ch0++; }
  while (frq>=1000000)
	{ frq=frq-1000000; ch1++; }
  while (frq>=100000)
	{ frq=frq-100000; ch2++; }
  while (frq>=10000) 
	{ frq=frq-10000; ch3++; }
  while (frq>=1000) 
	{ frq=frq-1000; ch4++; }
  while (frq>=100) 
	{ frq=frq-100; ch5++; }
  while (frq>=10) 
	{ frq=frq-10; ch6++; }
  ch7=frq+0x30;   										// konverter rest til ascii

	wrtlcdchr(ch0); // enkelte
	wrtlcdchr(ch1); // karakterer
	wrtlcdchr(','); // i
	wrtlcdchr(ch2); // display
	wrtlcdchr(ch3); // adskilt
	wrtlcdchr(ch4); // med
	wrtlcdchr('.'); // 
	wrtlcdchr(ch5); // (punktum)
	wrtlcdchr(ch6); // for
	wrtlcdchr(ch7); // at
	wrtlcdchr(' '); // forbedre
	wrtlcdchr('M'); // lsning
	wrtlcdchr('H'); // af
	wrtlcdchr('z'); // frekvensen
}
// END of wrtDEC **************************************************

// ****************************************************************
// Set LCD-display  til linie 2 og udskriv...
// ****************************************************************
void udskrivFRQ(unsigned long frq)
{
	sndlcd(0x46+0x80,'0');						// set LCD-pointer til 2. linie 
	wrtDEC(frq);
} 
// END of udskrivFRQ **********************************************

// ****************************************************************
// Konverter MF-frekvensen fra DECIMAL til LCD-display - HETERODYN MF 
// ****************************************************************
void udskrivMF(unsigned long frq)
{
	sndlcd(0x14+0x80,'0');						// set LCD-pointer til 3. linie sndlcd(0x02,'0');
	wrtlcdchr('M');
	wrtlcdchr('F');
	wrtlcdchr('=');
	wrtlcdchr(' ');
	wrtlcdchr(' ');
	wrtlcdchr(' ');
	wrtDEC(frq);
}
// END of udskrivMF ***********************************************

// ***************************************************************
// **********        Setup port til programbrug        ***********
// ***************************************************************
void initialPorts(void)
{
	ATRIS &= 0XE0;    							//alle 5 frste porte sat som output-porte i eet hug!

	scanTRIS     		= 1; 					//input VRIGE PORTE
	stepTRIS     		= 1;
	bandTRIS     		= 1;
	zbandTRIS    		= 1;
	UPstepTRIS      	= 1;
	DOWNstepTRIS      	= 1;
	StandardVFOTRIS    	= 1;
	SWEEPbandTRIS   	= 1;
	sweepTRIS   	 	= 1;
	esweepTRIS   		= 1;
	ZEROALLTRIS  		= 1; 
	ROTATORaTRIS 		= 1;
	ROTATORbTRIS 		= 1;

INTCON2bits.RBPU 		= 1; 							// Setup IRQ
	ADCON1           	= 0x0f;
	UCON             	= 0;
	UCFG             	= 0;
														// sluk alle dioder
PORTA = 0b00011111; 									// Alle porte p een gang!
}
// ********** END Setup port til programbrug ***********

// ***************************************************************
// Knap-funktion nr 6 (StandardVFO)
// ***************************************************************
void HeterodynMF(void)
{
	sndlcd(0x00+0x80,'0');								// move LCD to pos 0
	wrtrlcd((char rom *) "VFO-C Set MF OFFset ",0);

	if ( StandardVFO == knapON ) 					// er "MF-OFFset"-knap sluppet ?
	{
		Delay10KTCYx(dell);							// PAUSE hvis knappen stadig er trykket
		if ( StandardVFO == knapON )				// stadig trykket efter 10 milisec. ?
		{ Delay10KTCYx(dell); }
	}

	
	HeroMF = frequens + tmpSSB ;
	udskrivMF(HeroMF);
	MyDelay(100);
	SigGen();
}

// *************************************************************
// **  set filterbndbredde for SSB, for Standard VFO
// **  TOOGLES ON/OFF med knap 9
// ** Frste gang st
// **    ssb = 0		SSB>USB		SET SSB = 1
// ** 2. SSB = 1		SSS>LSB		SET SSB = 2
// ** 3. SSB = 2		SSB>CW		SET SSB = 3
// ** 4. SSB = 3		SSB>0FF		SET SSB = 0      
// *************************************************************
void setMODE(void)
{
tmpSSB = 0;
		sndlcd(0x5d+0x80,'0');
		wrtlcdchr('S');
		wrtlcdchr('S');
		wrtlcdchr('B');
		wrtlcdchr('=');
		wrtlcdchr('O');
switch ( SSB )
	{
	case 0: 
		SSB = 1;										// SET SSB = 1
		tmpSSB = USB;									// tmpSSB SAT
		wrtlcdchr('N');
		wrtlcdchr(' '); 
		wrtlcdchr(' ');
		wrtlcdchr('U');
		wrtlcdchr('S'); 
		wrtlcdchr('B');
		break;
	case 1:
		SSB = 2;										// SET SSB = 2
		tmpSSB = LSB;									// tmpSSB SAT
		wrtlcdchr('N');
		wrtlcdchr(' '); 
		wrtlcdchr(' ');
		wrtlcdchr('L');
		wrtlcdchr('S');
		wrtlcdchr('B');
		break;
	case 2:
		SSB = 3;										// SET SSB = 2
		tmpSSB = CW;									// tmpSSB SAT
		wrtlcdchr('F');
		wrtlcdchr('F'); 
		wrtlcdchr(' ');
		wrtlcdchr('C');
		wrtlcdchr('W');
		wrtlcdchr(' ');
		break;
	case 3:
		SSB = 0;
		wrtlcdchr('F');
		wrtlcdchr('F'); 
		wrtlcdchr(' ');
		wrtlcdchr(' ');
		wrtlcdchr(' ');
		wrtlcdchr(' ');
	}
	HeterodynMF();
	MyDelay(100);
}
// END of setMODE ************************************************


// *********** END OF PROGRAM *************************************

