// ***********************************************************
// LCD Display driver - STANDARD 4 line x 20 Char display
// Forbindelse mellem CPU = PIC18F4553 og LDC,
// 		defineret herunder, er:
//
//				LCD				CPU				SIGNAL
//     ---------------------------------------------------
//			pin 4 - RS		=	pin 19 - RD0 = Register Select
//			pin 5 - R/W		=	pin 20 - RD1 - READ/WRITE
//			pin 6 - E		= 	pin 21 - RD2 - Enable
//									  (  RD3 - 'zeroall' )
//
//			pin 7,8,9,10	=	NC			 - Not Connected
// 4 bit DATA
//			pin 11			=	pin 25 - RD4 - DATA
//			pin 12 			=   pin 26 - RD5 - DATA
//			pin 13			=   pin 27 - RD6 - DATA
// 			pin 14			= 	pin 28 - RD7 - DATA
// Rettet den 24.12.2009
// ************************************************************
//
// Control Header Defines  
// 
#include "lcd.h"
#include <string.h>
#include "p18f4553.h"

#define mInitAllLCDs()      LATD &= 0x00; TRISD &= 0x00; 
#define	LCDRS	LATDbits.LATD0   	// RS Pin 
#define	LCDRW	LATDbits.LATD1   	// RW Pin
#define	LCDENA	LATDbits.LATD2  	// E pin 

#define LCDBIT7 PORTDbits.RD7 		// Data 7
#define LCDBIT6 PORTDbits.RD6 		// Data 6
#define LCDBIT5 PORTDbits.RD5 		// Data 5
#define LCDBIT4 PORTDbits.RD4 		// Data 4

#define LCDBIT4TRIS TRISDbits.TRISD4 // Tris 
#define LCDBIT5TRIS TRISDbits.TRISD5 // Tris  
#define LCDBIT6TRIS TRISDbits.TRISD6 // Tris 
#define LCDBIT7TRIS TRISDbits.TRISD7 // Tris  

#define LCDIOTIME 1 
#define ClrWdt() {_asm clrwdt _endasm}

char lcdoldpos;



//
// ***** Loop Delay *******************************************
// 				NOTE:	This delay is not using timer, 
// 						but based on CPUCLOCK and ClockCycle
// ************************************************************
void Del(unsigned int del0)    		// input in 4uS           
{ 
int FastDel= del0 ; 				// 4*CPUCLOCK/(ClockCycle*17) 1 = 16MHz, 2 = 32MHz
 while (FastDel>=1)  				// takes 17 Cycels
  { 
    ClrWdt();  						// takes 1 Cycle
   FastDel--;
  } 								// Total 18 Cycles per Loop
}
//
// ************************************************************
// ******   Clear alle 4 linier                           *****
// ************************************************************
void ClearLCD(void)
{
	sndlcd(0x00+0x80,'0');			// move LCD to pos 0
	wrtrlcd((char rom *) "                    ",0);
	wrtrlcd((char rom *) "                    ",1);
	wrtrlcd((char rom *) "                    ",2);
    wrtrlcd((char rom *) "                    ",3);
	sndlcd(0x40+0x80,'0'); // her er et par felter som osse skal 'Cleares'
    wrtlcdchr(' ');
	sndlcd(0x00+0x80,'0');
	wrtlcdchr(' ');
}
//
// ************************************************************
// LCD service rutines
// J. Soya - Fiasko Software - Aug 2006
// 3024 Mockup - 4 Bit LCD access
// ************************************************************ 
// System use
// sndLCD(cmd,rsrw) - operate LCD interface and wait for LCD to process
// iniLCD() - Inititialize LCD
// wrtLCD(*tekst) - Writes 20 chars on physical LCD
// chgLCD() - Timer swaped display
// ******** Basic LCD I/O ****************************
// cmd id Data byte
// rsrw - See just below
// ************************************************************
void lcdsetctl(unsigned char rsrw)
{
if (rsrw == '0') 					//	Set control Bit's - 
	{	LCDRS = 0;
		LCDRW = 0;}
	if (rsrw == '1')
	{	LCDRS = 0;
		LCDRW = 1;}
	if (rsrw == '2')
	{	LCDRS = 1;
		LCDRW = 0;}
	if (rsrw == '3')
	{	LCDRS = 1;
		LCDRW = 1;}
	if (rsrw == '4') 				// Initializing.. no BF test
	{	LCDRS = 0;
		LCDRW = 0;}
	if (rsrw == '5') 				// Initializing.. 4 bit and no BF test
	{	LCDRS = 0;
		LCDRW = 0;}
	if (rsrw == 'F') 				// Read BF & Address
	{	LCDRW = 1;					// Set read
 		LCDRS = 0;	}				// set read
}
//
// ************************************************************
void lcdsethighnibble(unsigned char cmd)
{
	if (0x80 & cmd) 
			{	LCDBIT7  = 1;
			} 						// One
			else
			{	LCDBIT7  = 0;
			} 						// Zero
		if (0x40 & cmd) 
			{	LCDBIT6  = 1;
			} 						// One
			else
			{LCDBIT6  = 0;
			} 						// Zero
		if (0x20 & cmd) 
			{	LCDBIT5  = 1;
			} 						// One
			else
			{	LCDBIT5 = 0;
			} 						// Zero
		if (0x10 & cmd) 
			{LCDBIT4  = 1;
			} 						// One
			else
			{LCDBIT4 = 0;
			} 						// Zero
}
//
// ************************************************************
void lcdsetlownibble(unsigned char cmd)
{
	if (0x08 & cmd) 
			{	LCDBIT7  = 1;
			} 						// One
			else
			{		LCDBIT7  = 0;
			} 						// Zero
		if (0x04 & cmd) 
			{	LCDBIT6  = 1;
			} 						// One
			else
			{LCDBIT6 = 0;
			} 						// Zero
		if (0x02 & cmd) 
			{	LCDBIT5 = 1;
			} 						// One
			else
			{	LCDBIT5 = 0;
			} 						// Zero
		if (0x01 & cmd) 
			{LCDBIT4 = 1;
			} 						// One
			else
			{	LCDBIT4 = 0;
			} 						// Zero
}
//
// ************************************************************
void waitlcdbusy(void)
{
									// Ready to read from Lcd
LCDBIT7TRIS = 1;					// Input Tris on BUSYFLAG (BF)

	Del(LCDIOTIME);  				// reaction time
  	lcdsetctl('F');
									// read BF High nibble
	Del(LCDIOTIME);  				// reaction time
 	LCDENA = 1;
	Del(LCDIOTIME);  				// reaction time
	while (LCDBIT7 ) 				//   Check BF
		{   						// Read Low Nibble	 	     
			Del(LCDIOTIME);  		// reaction time
 			LCDENA = 0;
									// read BF High nibble
			Del(LCDIOTIME);  		// reaction time
 			LCDENA = 1;
			Del(LCDIOTIME);  		// reaction time
	 	} 							// while end -- Retest
									//	if (LCDBIT7 ) asychr('B');		// This is a serious error
	Del(LCDIOTIME); 
									// get next
	LCDENA = 0;
	Del(LCDIOTIME); 
	LCDBIT7TRIS = 0;				// Output  Tris on BF
}
//
// ************************************************************
void sndlcd(unsigned char cmd, unsigned char rsrw)
{ 									// Fast command LCD 
char wrk;							// Ready to write LCD	
	lcdsetctl(rsrw);	
	Del(LCDIOTIME);
	Del(500);
									// Do the actual thing
	if (rsrw == '5')
		{							// Special 4 Bit setup, no write low nibble		 
		lcdsethighnibble(cmd);
		LCDENA = 1;					// Latch high	 
		Del(LCDIOTIME);				// 4 uS
		LCDENA = 0;					// Latch Data
		Del(LCDIOTIME);  			// reaction time ?? used to be 40	    
		} 							// end 4 bit init
		else
		{ 							// this is for all duble nibble writes
									// write first nibble
		lcdsethighnibble(cmd);		 
		Del(LCDIOTIME)	;			// 4 uS
		LCDENA = 1;					// Latch HIgh
		Del(LCDIOTIME);  			// reaction time ?? used to be 40
		LCDENA = 0;					// Latch data 
		Del(LCDIOTIME);  			// reaction time ?? used to be 40
									// write last nibble
		lcdsetlownibble(cmd);
		Del(LCDIOTIME)	;			// 4 uS
		LCDENA = 1;					// Latch High
		Del(LCDIOTIME);  			// reaction time ?? used to be 40		
		LCDENA = 0;					// Latch data
		Del(LCDIOTIME);  			// reaction time ?? used to be 40		
		}
									// Poll for finished
	if (rsrw <= '3') 				// These can use wait for Busy FLag
	{
	LCDBIT4TRIS = 1;
	LCDBIT5TRIS = 1;
	LCDBIT6TRIS = 1;
 	LCDBIT7TRIS = 1;				// Input Tris on BF
	Del(LCDIOTIME);  				// reaction time
  	lcdsetctl('F');
									// read BF High nibble
	Del(LCDIOTIME);  				// reaction time
 	LCDENA = 1;
	Del(LCDIOTIME);  				// reaction time
	while (LCDBIT7 ) 				//   Check BF
		{   						// Read Low Nibble	 	     
			Del(LCDIOTIME);  		// reaction time
 			LCDENA = 0;
									// read BF High nibble
			Del(LCDIOTIME);  		// reaction time
 			LCDENA = 1;
			Del(LCDIOTIME);  		// reaction time
	 	} 							// while end -- Retest
									//	if (LCDBIT7 ) asychr('B');		// This is a serious error
									// Read top 3 bits
	wrk = 0x00;
	if (LCDBIT6) wrk = wrk ^ 0x40;
	if (LCDBIT5) wrk = wrk ^ 0x20;
	if (LCDBIT4) wrk = wrk ^ 0x10;
	Del(LCDIOTIME); 
									// get next
	LCDENA = 0;
	Del(LCDIOTIME); 
 									// read BF low nibble
 	LCDENA = 1;
	Del(LCDIOTIME);					// do a break time	
	if (LCDBIT7) wrk = wrk ^ 0x08;
	if (LCDBIT6) wrk = wrk ^ 0x04;
	if (LCDBIT5) wrk = wrk ^ 0x02;
	if (LCDBIT4) wrk = wrk ^ 0x01;
 	lcdoldpos = wrk ;
 	LCDBIT4TRIS = 0;
	LCDBIT5TRIS = 0;
	LCDBIT6TRIS = 0;
 	LCDBIT7TRIS = 0;				// Input Tris on BF // Ready to write LCD
	} 								// end if do BF Address
									// on exit ENA = 1, RS = 0, RW = 0
}
// ********* Initialize LCD ***********************************
// *** Write LCD with Text String *****************************
void wrtlcd(char *tekst,char pos)	// writes text to LCD 20 position 1 line dislay
{ 	char lcdptr = 0;				// set position 0
 	sndlcd(0x02,'0');				// move LCD to pos 0
 	switch(pos)
		{
		case 0:
			sndlcd(0x00+0x80,'0');	// move LCD to pos 
			break;

		case 1:
			sndlcd(0x40+0x80,'0');	// move LCD to pos  
			break;

		case 2:
			sndlcd(0x14+0x80,'0');	// move LCD to pos  
			break;

		case 3:
			sndlcd(0x54+0x80,'0');	// move LCD to pos  
			break;
		}
									// Zero pointer
 	while (lcdptr <= 21)
	{ 
	 	if ((char) '\0'  != tekst[lcdptr])	// change nulls to blanks
			{
				sndlcd(tekst[lcdptr],'2');	// Send char
			} 
 		++lcdptr;							// update pointer		 							
 	}
}
//
// ************************************************************
// void wrtrlcd( const rom char * tekst,char pos)
// ************************************************************
void wrtrlcd(const rom char * tekst,char pos) // og husk at rette prototypen i "H"
{
	char a[21];
	strncpypgm2ram((char *)a,(const rom char * )tekst,21);
	wrtlcd(a,pos);
}
//
// ************************************************************
void wrtlcdchr(char a)				//
{
		sndlcd(a,'2');				// Send char
}
//
// ************************************************************
// ************************************************************
void iniLCD(void)
{	
// initialization for 20 position 4 line dislay 4 Bit interface
// start 4 Bit interface startet
char init[4][21] = {
	{"                    "},	// clear all 4 lines
	{"                    "},
	{"                    "},
	{"                    "},
	};					
	mInitAllLCDs();
	LCDENA = 0;						// Data Latch Data
	Del(500);						// Just in case 
	sndlcd(0x30,'5');    			// 4 Bit interface
 	Del(200);
	sndlcd(0x30,'5');    			// 4 Bit interface
 	Del(200);
	sndlcd(0x20,'5');    			// 4 Bit interface
 	Del(200);	
									// 4 Bit interface startet
	sndlcd(0x2c,'4');				// Function - 1' line & Display on
 	Del(100);
	waitlcdbusy();
 	sndlcd(0x0c,'4');				// Display On, Cursor ON, Blonk OFF
 	Del(100);	
	waitlcdbusy();
	sndlcd(0x01,'4');				// Display Clear
	Del(500);
 	waitlcdbusy();
	sndlcd(0x06,'4'); 				// Entry set
	Del(100);
 	waitlcdbusy();
	ClearLCD();
	sndlcd(0x00+0x80,'0');			// move LCD to pos 0
	wrtrlcd((char rom *) "  SDRadio  Project  ",0);
	wrtrlcd((char rom *) "       DDS-VFO      ",1);
	wrtrlcd((char rom *) "  + Signalgenerator ",2);
    wrtrlcd((char rom *) "     2013 Ver: 4.3  ",3);
	sndlcd(0x40+0x80,'0'); // et par bit mere skal CLEARES
    wrtlcdchr(' ');
	sndlcd(0x00+0x80,'0');
	wrtlcdchr(' ');
									// Lad lige tiden g lidt...
}
// 




