/********************************************************************
 FileName:      main.c
 Dependencies:  See INCLUDES section
 Processor:		PIC18F4553
 Hardware:		SDRADIO-VFO - OZ5WF-project
 Complier:  	Microchip C18 (for PIC18)
 Company:		Microchip Technology, Inc.

********************************************************************
 File Description:
********************************************************************/

/** INCLUDES *******************************************************/
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "delays.h"
#include "HardwareProfile.h"
#include "stdio.h"
#include "stdlib.h"
#include "math.h"

/***************  My INCLUDES ************************************/
#include "lcd.h" 
#include "DDS.h"
#include "Porte.h"

/** CONFIGURATION **************************************************/

//**    MY     V A R I A B L E S ***********************************************
extern void DDSup(void);
extern void DDSdown(void);
extern void moveFrekvens(int);
extern void Stepskifte(void);
extern void WRTpoint();
extern void set_bandPTR(void);
extern void HeterodynMF(void);

extern void timer1trig(void);
extern void setFrekvens(void);
extern void zeroCurrent(void);
extern void SigGen(void);
extern void EndSigGen(void);
extern void MySweep(void);
extern void ClearLCD(void);
extern void WRTstep(unsigned int);
extern void updateDDS(void);
extern void MyDelay(unsigned int del0);
extern void setMODE(void);

extern unsigned long freq_step;
extern unsigned long freq_frm;	// sweep from
extern unsigned long freq_to;
extern int menuflag;		// KnapTest-Flag
extern int bandPTR;
int ksFlag;					// KnapSwitch-flag
int del0 = 0;
signed short step1; //int step1;
signed short MyStep; //int MyStep;
int XN;
signed short step; 						/* rotation since reset */
char unit = 1;
static volatile unsigned char ab = 0; 	/* the old value of the sensor ports */

// **********************************************************************
// const signed short table[] = {0,-1,1,0, 1,0,0,-1, -1,0,0,1, 0,1,-1,0};
const signed short table[] = {0,-1,1,0, 1,0,0,-1, -1,0,0,1, 0,1,-1,0};
volatile signed char rotatorcount = 0; 
// This is the place to look for changes since last check ***PALLE
// **********************************************************************

// ***PALLE
// *****Definition for timer-setup, tick, counter, byte high and byte low *****
#define TICK	1;							// ms per tick (max T1COUNT = 60000 ca. lig 40 ms.
#define T1COUNT 1500 * TICK					// 1 ms takes 1500 clocks of 1.5 Mhz
#define PALLES_H_COUNT 5 					// (T1COUNT / 256)		// 1 ms = 5
#define PALLES_L_COUNT T1COUNT - ( PALLES_H_COUNT * 256 ) 	// 1 ms = 220


/** P R I V A T E  P R O T O T Y P E S ***************************************/

void YourHighPriorityISRCode();
void UserInit(void);
void Delay1KTCYx( unsigned char unit );

extern unsigned long CurStartFrekvens;
extern unsigned long CurFrekvens;
extern unsigned long CurEndFrekvens;
extern int Cur_bandPTR;

// 
// ********************************************************************************
// Delay Functions:
//
// Prototype:  void Delay1TCY( void );
// 		Function  Description  
// Delay1TCY 		= Delay one instruction cycle.
//      This function is actually a #define for the NOP instruction. 
//      When encountered in the source code, the compiler simply inserts a NOP.
//
// Prototype:  void Delay10TCYx( unsigned char unit );   
// Delay10TCYx 		= Delay in multiples of 10 instruction cycles.  
//
// Prototype:  void Delay100TCYx( unsigned char unit );
// Delay100TCYx 	= Delay in multiples of 100 instruction cycles.
//
// Prototype:  void DelayKTCYx( unsigned char unit );  
// Delay1KTCYx 		= Delay in multiples of 1,000 instruction cycles.  
//
// Prototype:  void Delay10KTCYx( unsigned char unit );
// Delay10KTCYx 	= Delay in multiples of 10,000 instruction cycles.  
//
// ****************************************
// GENEREL LOOP-DELAY - (midlertidig routine)
void MyDelay(unsigned int del0) // input in 4uS           
{ 								// NOTE - this delay is not using timer, 
 Delay10KTCYx(del0);			// but based on CPUCLOCK and ClockCycle
 Delay10KTCYx(del0);			// ADDED med #include delays.h
 Delay10KTCYx(del0);			// se MCC18 - help-file for Libraries Overview  
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
 Delay10KTCYx(del0);
}
// ***** END LOOP-DELAY *******************

/** VECTOR REMAPPING ***********************************************/

#if defined(__18CXX)
	//On PIC18 devices, addresses 0x00, 0x08, and 0x18 are used for
	//the reset, high priority interrupt, and low priority interrupt
	//vectors.  However, the current Microchip USB bootloader 
	//examples are intended to occupy addresses 0x00-0x7FF or
	//0x00-0xFFF depending on which bootloader is used.  Therefore,
	//the bootloader code remaps these vectors to new locations
	//as indicated below.  This remapping is only necessary if you
	//wish to program the hex file generated from this project with
	//the USB bootloader.  If no bootloader is used, edit the
	//usb_config.h file and comment out the following defines:
	//#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER
	//#define PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018

	#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818

	#else	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
	#endif
	
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS


	void Remapped_High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)
	{

	}
	
	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#endif	//end of "#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER)"

	#pragma code
	
	
	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{ 	//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flagja, som af een eller anden grund ikke optrder p scenen....

		//Etc.

		// Interrupt code ***PALLE***
		if (PIR1bits.TMR1IF)
			{ // This does something if TMR1 is triggered
 			  TMR1H = 0xFF - PALLES_H_COUNT;	// Counter clocks up - hit zero --> do interrupt
			  TMR1L = TMR1L - PALLES_L_COUNT;			  
			  timer1trig(); // Call Palles Timer activity			 
			  PIR1bits.TMR1IF = 0; // Clear Interrupt flag

			}
	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
/*
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
	
	}	//This return will be a "retfie", since this is in a #pragma interruptlow section 
*/
/** DECLARATIONS ***************************************************/
#pragma code

/******************************************************************************/
/*********************** Interrupt servie for timer1 **************************/
void timer1trig(void)
{
// this is check on rotator
/******************************************************************************/
ab = ab << 2; 						/* move the old data left two places */
ab |= (PORTB & 0x3); 				/* OR in the two new bits */
rotatorcount += table[(ab & 0xf)];  /* get the change from the 16 entry table */
}
/******************************************************************************/

/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
void main(void)
{
UserInit();

// ***********************  program  ****************************
initialPorts();
iniLCD();
MyDelay(500);
DDSsetup();
Cur_bandPTR = 1; // temp initiated
zeroCurrent();
ClearLCD();
step = 4;
Stepskifte();	

// her starter det egentlige program*****
//***************************************
bandPTR = Cur_bandPTR;
set_bandPTR();
setFrekvens();

startMenu:
/* rotation since reset */
//static unsigned char ab = 0; 	/* the old value of the sensor ports */
//const signed short table[] = {0,-1,1,0, 1,0,0,-1, -1,0,0,1, 0,1,-1,0}; */
// increment of angle for the 16 possible bit codes */
// ---------------------------------------------------
// Rotatorcheck:		// Rotator = port RB0 og RB1 ~ %00000011 = 3 (dec)
INTCONbits.GIE = 0;		// turn off interrupts
	step1 = rotatorcount;
	rotatorcount = 0;
INTCONbits.GIE = 1;		// turn ON interrupts
// ***************************************************

updateDDS();

KnapScan();
switch (ksFlag)
	{
		case 0: break;				// 0 forekommer IKKE

		case 1: Stepskifte();		// Udfr stepskifte
				ksFlag = 0;
				break;

		case 2: setFrekvens();		// Udfr Skift Frekvensbnd
				ksFlag = 0;
				break;

		case 3: zeroCurrent();		// Nulstil frekvenspionter til current bndstart
				ksFlag = 0;
				break;

		case 4: DDSup();			// forg current freq. med step-pointer
				ksFlag = 0;
				MyDelay(5);
				break;

		case 5: DDSdown();			// formindsk current freq. med step-pointer
				ksFlag = 0;
				MyDelay(5);
				break;

		case 6: HeterodynMF();		// Set MF for hetorodyn-modtager
				ksFlag = 0;
				break;

		case 7: MySweep();			// Signalgenerator/MF Sweep 	
				ksFlag = 0;
				break;

		case 8: SigGen(); 			// Vlg og indstil heterodyn MF
				ksFlag = 0;  
				break;

		case 9: setMODE(); 			// 
				ksFlag = 0;
				break;

		case 10: EndSigGen();
				ksFlag = 0;
				break;
	}
// **************************        END main loop  /
goto startMenu;


}	//end main


/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:            
 *
 *****************************************************************************/
void UserInit(void)
{

// ***PALLE
// setup TMR1 for interrupts
// T1CONbits.T1CKPS1 = 1;		// Prescale = 8
// T1CON
T1CONbits.RD16 = 0;			// write in to operations ... can be better with 16.. who cares
T1CONbits.T1RUN = 0;  		// cut the internal timer1 oscillator
T1CONbits.T1CKPS1 = 1;		// giving count at 48/8/4 = 1.5 Mhz
T1CONbits.T1OSCEN = 0;		// Enable Timer
T1CONbits.T1SYNC = 1;		// No ext sync
T1CONbits.TMR1CS = 0;		// FOSC / 4 - T1CONbit.TMR1CS = 0; 		// use master clock as clock
T1CONbits.TMR1ON = 1;		// please run

// Interrupt
PIE1bits.TMR1IE = 1;		// do interrupts
INTCONbits.GIE = 1;			// turn ON interrupts
INTCONbits.PEIE = 1;		// turn ON interrupts

}//end UserInit

/******************************************************************************

/** EOF main.c *************************************************/
